# aplicação que mostra os conceitos de análise de regressão
# autor: Sergio Miranda Freire

library(shiny)
library(plotly)
library(rgl)
library(plot3D)

data(Prestige, package="car")
x <- Prestige$education
y <- Prestige$prestige
        
server <- function(input, output) {

    a <- reactive ({
        input$alfa
    })

    b <- reactive ({
        input$beta
    })
    
    output$retaRegressao <- renderPlot({
        plot(x, y, xlab = "Nível educacional", ylab = "Prestígio", xlim = c(0, 17), ylim = c(-15, 100), col = "blue")
        abline(a(), b(), lwd = 2)
        segments(x, y, x, a() + b() * x, col = "red")
        sqerror <- sum((y - (a() + b() * x))^2) / (length(x) - 2)
        par(cex = 1.5)
        text(8, 80, paste("Erro quadrático médio (EQM)= ", sprintf("%.1f", sqerror)))
        
    })    
    
    output$superficieErro <- renderPlotly({
        alpha <- sort(c(seq(-15, 0, length.out = 100), -10.732))
        beta <- sort(c(seq(4.5, 6, length.out = 100), 5.361))
        z <- matrix(nrow = length(alpha), ncol = length(beta))
        for (i in 1:length(alpha)) {
            for (j  in 1:length(beta)) {
                z[i,j] <- sum( (y - (alpha[i]+beta[j]*x) )^2 ) / (length(x) - 2)
            }
        }
        M <- mesh(alpha, beta)
        x <- M$x
        y <- M$y
        plot_ly(x = x, y = y, z = z, type = "surface") %>%
        layout(scene = list(
            xaxis = list(title = "interseção"),
            yaxis = list(title = "inclinação"),
            zaxis = list(title = "EQM")
        ))
    })    
}


# Define UI for calculate the statistical power and the effects 
# of sample size on statistical power
ui <- fluidPage(
  
  # Application title
  titlePanel(paste(expression(Determinação), "dos coeficientes da reta de ", expression(regressão))),
  
  sidebarLayout(
    sidebarPanel(
        sliderInput("alfa", 
                    paste(expression(Interseção), " da reta:"), 
                    min = -20, 
                    max = 15, 
                    step = .001,
                    value = -11
        ),
        sliderInput("beta", 
                paste(expression(Inclinação), " da reta:"), 
                min = 0, 
                max = 7, 
                step = .001,
                value = 4.5
        )
  ),
  
    # Show plots
    mainPanel(
        fluidRow(
            column(8, plotOutput("retaRegressao")),
            column(8, plotlyOutput("superficieErro"))
        )
    )
)
)

shinyApp(ui = ui, server = server)